@@ -23,9 +23,7 @@
 
 #include "P_Cache.h"
 
-#ifdef HTTP_CACHE
 #include "HttpCacheSM.h" //Added to get the scope of HttpCacheSM object.
-#endif
 
 extern int cache_config_compatibility_4_2_0_fixup;
 
@@ -56,14 +54,16 @@ Cache::open_read(Continuation *cont, const CacheKey *key, CacheFragType type, co
       c->frag_type                            = type;
       c->od                                   = od;
     }
-    if (!c)
+    if (!c) {
       goto Lmiss;
+    }
     if (!lock.is_locked()) {
       CONT_SCHED_LOCK_RETRY(c);
       return &c->_action;
     }
-    if (c->od)
+    if (c->od) {
       goto Lwriter;
+    }
     c->dir            = result;
     c->last_collision = last_collision;
     switch (c->do_read_call(&c->key)) {
@@ -81,19 +81,20 @@ Lmiss:
   return ACTION_RESULT_DONE;
 Lwriter:
   SET_CONTINUATION_HANDLER(c, &CacheVC::openReadFromWriter);
-  if (c->handleEvent(EVENT_IMMEDIATE, nullptr) == EVENT_DONE)
+  if (c->handleEvent(EVENT_IMMEDIATE, nullptr) == EVENT_DONE) {
     return ACTION_RESULT_DONE;
+  }
   return &c->_action;
 Lcallreturn:
-  if (c->handleEvent(AIO_EVENT_DONE, nullptr) == EVENT_DONE)
+  if (c->handleEvent(AIO_EVENT_DONE, nullptr) == EVENT_DONE) {
     return ACTION_RESULT_DONE;
+  }
   return &c->_action;
 }
 
-#ifdef HTTP_CACHE
 Action *
-Cache::open_read(Continuation *cont, const CacheKey *key, CacheHTTPHdr *request, CacheLookupHttpConfig *params, CacheFragType type,
-                 const char *hostname, int host_len)
+Cache::open_read(Continuation *cont, const CacheKey *key, CacheHTTPHdr *request, OverridableHttpConfigParams *params,
+                 CacheFragType type, const char *hostname, int host_len)
 {
   if (!CacheProcessor::IsCacheReady(type)) {
     cont->handleEvent(CACHE_EVENT_OPEN_READ_FAILED, (void *)-ECACHE_NOT_READY);
@@ -126,10 +127,12 @@ Cache::open_read(Continuation *cont, const CacheKey *key, CacheHTTPHdr *request,
       CONT_SCHED_LOCK_RETRY(c);
       return &c->_action;
     }
-    if (!c)
+    if (!c) {
       goto Lmiss;
-    if (c->od)
+    }
+    if (c->od) {
       goto Lwriter;
+    }
     // hit
     c->dir = c->first_dir = result;
     c->last_collision     = last_collision;
@@ -151,23 +154,25 @@ Lwriter:
   // this is a horrible violation of the interface and should be fixed (FIXME)
   ((HttpCacheSM *)cont)->set_readwhilewrite_inprogress(true);
   SET_CONTINUATION_HANDLER(c, &CacheVC::openReadFromWriter);
-  if (c->handleEvent(EVENT_IMMEDIATE, nullptr) == EVENT_DONE)
+  if (c->handleEvent(EVENT_IMMEDIATE, nullptr) == EVENT_DONE) {
     return ACTION_RESULT_DONE;
+  }
   return &c->_action;
 Lcallreturn:
-  if (c->handleEvent(AIO_EVENT_DONE, nullptr) == EVENT_DONE)
+  if (c->handleEvent(AIO_EVENT_DONE, nullptr) == EVENT_DONE) {
     return ACTION_RESULT_DONE;
+  }
   return &c->_action;
 }
-#endif
 
 uint32_t
 CacheVC::load_http_info(CacheHTTPInfoVector *info, Doc *doc, RefCountObj *block_ptr)
 {
   uint32_t zret = info->get_handles(doc->hdr(), doc->hlen, block_ptr);
-  if (cache_config_compatibility_4_2_0_fixup && // manual override not engaged
-      !this->f.doc_from_ram_cache &&            // it's already been done for ram cache fragments
-      vol->header->version.ink_major == 23 && vol->header->version.ink_minor == 0) {
+  if (!this->f.doc_from_ram_cache && // ram cache is always already fixed up.
+                                     // If this is an old object, the object version will be old or 0, in either case this is
+                                     // correct. Forget the 4.2 compatibility, always update older versioned objects.
+      VersionNumber(doc->v_major, doc->v_minor) < CACHE_DB_VERSION) {
     for (int i = info->xcount - 1; i >= 0; --i) {
       info->data(i).alternate.m_alt->m_response_hdr.m_mime->recompute_accelerators_and_presence_bits();
       info->data(i).alternate.m_alt->m_request_hdr.m_mime->recompute_accelerators_and_presence_bits();
@@ -196,8 +201,9 @@ CacheVC::openReadChooseWriter(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSE
 
   ink_assert(vol->mutex->thread_holding == mutex->thread_holding && write_vc == nullptr);
 
-  if (!od)
+  if (!od) {
     return EVENT_RETURN;
+  }
 
   if (frag_type != CACHE_FRAG_TYPE_HTTP) {
     ink_assert(od->num_writers == 1);
@@ -206,26 +212,28 @@ CacheVC::openReadChooseWriter(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSE
       od = nullptr;
       return EVENT_RETURN;
     }
-    if (!w->closed)
+    if (!w->closed) {
       return -err;
+    }
     write_vc = w;
-  }
-#ifdef HTTP_CACHE
-  else {
+  } else {
     write_vector      = &od->vector;
     int write_vec_cnt = write_vector->count();
-    for (int c = 0; c < write_vec_cnt; c++)
+    for (int c = 0; c < write_vec_cnt; c++) {
       vector.insert(write_vector->get(c));
+    }
     // check if all the writers who came before this reader have
     // set the http_info.
     for (w = (CacheVC *)od->writers.head; w; w = (CacheVC *)w->opendir_link.next) {
-      if (w->start_time > start_time || w->closed < 0)
+      if (w->start_time > start_time || w->closed < 0) {
         continue;
+      }
       if (!w->closed && !cache_config_read_while_writer) {
         return -err;
       }
-      if (w->alternate_index != CACHE_ALT_INDEX_DEFAULT)
+      if (w->alternate_index != CACHE_ALT_INDEX_DEFAULT) {
         continue;
+      }
 
       if (!w->closed && !w->alternate.valid()) {
         od = nullptr;
@@ -240,14 +248,15 @@ CacheVC::openReadChooseWriter(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSE
         alt_ndx = get_alternate_index(&vector, w->update_key);
         // if its an alternate delete
         if (!w->alternate.valid()) {
-          if (alt_ndx >= 0)
+          if (alt_ndx >= 0) {
             vector.remove(alt_ndx, false);
+          }
           continue;
         }
       }
-      ink_assert(w->alternate.valid());
-      if (w->alternate.valid())
+      if (w->alternate.valid()) {
         vector.insert(&w->alternate, alt_ndx);
+      }
     }
 
     if (!vector.count()) {
@@ -262,10 +271,12 @@ CacheVC::openReadChooseWriter(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSE
     }
     if (cache_config_select_alternate) {
       alternate_index = HttpTransactCache::SelectFromAlternates(&vector, &request, params);
-      if (alternate_index < 0)
+      if (alternate_index < 0) {
         return -ECACHE_ALT_MISS;
-    } else
-      alternate_index  = 0;
+      }
+    } else {
+      alternate_index = 0;
+    }
     CacheHTTPInfo *obj = vector.get(alternate_index);
     for (w = (CacheVC *)od->writers.head; w; w = (CacheVC *)w->opendir_link.next) {
       if (obj->m_alt == w->alternate.m_alt) {
@@ -283,7 +294,6 @@ CacheVC::openReadChooseWriter(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSE
     DDebug("cache_read_agg", "%p: key: %X eKey: %d # alts: %d, ndx: %d, # writers: %d writer: %p", this, first_key.slice32(1),
            write_vc->earliest_key.slice32(1), vector.count(), alternate_index, od->num_writers, write_vc);
   }
-#endif // HTTP_CACHE
   return EVENT_NONE;
 }
 
@@ -310,16 +320,18 @@ CacheVC::openReadFromWriter(int event, Event *e)
     return free_CacheVC(this);
   }
   CACHE_TRY_LOCK(lock, vol->mutex, mutex->thread_holding);
-  if (!lock.is_locked())
+  if (!lock.is_locked()) {
     VC_SCHED_LOCK_RETRY();
+  }
   od = vol->open_read(&first_key); // recheck in case the lock failed
   if (!od) {
     MUTEX_RELEASE(lock);
     write_vc = nullptr;
     SET_HANDLER(&CacheVC::openReadStartHead);
     return openReadStartHead(event, e);
-  } else
+  } else {
     ink_assert(od == vol->open_read(&first_key));
+  }
   if (!write_vc) {
     int ret = openReadChooseWriter(event, e);
     if (ret < 0) {
@@ -337,8 +349,9 @@ CacheVC::openReadFromWriter(int event, Event *e)
       } else {
         return openReadFromWriterFailure(CACHE_EVENT_OPEN_READ_FAILED, (Event *)-err);
       }
-    } else
+    } else {
       ink_assert(write_vc);
+    }
   } else {
     if (writer_done()) {
       MUTEX_RELEASE(lock);
@@ -349,10 +362,8 @@ CacheVC::openReadFromWriter(int event, Event *e)
       return openReadStartHead(event, e);
     }
   }
-#ifdef HTTP_CACHE
   OpenDirEntry *cod = od;
-#endif
-  od = nullptr;
+  od                = nullptr;
   // someone is currently writing the document
   if (write_vc->closed < 0) {
     MUTEX_RELEASE(lock);
@@ -381,24 +392,26 @@ CacheVC::openReadFromWriter(int event, Event *e)
   }
   MUTEX_RELEASE(lock);
 
-  if (!write_vc->io.ok())
+  if (!write_vc->io.ok()) {
     return openReadFromWriterFailure(CACHE_EVENT_OPEN_READ_FAILED, (Event *)-err);
-#ifdef HTTP_CACHE
+  }
   if (frag_type == CACHE_FRAG_TYPE_HTTP) {
     DDebug("cache_read_agg", "%p: key: %X http passed stage 1, closed: %d, frag: %d", this, first_key.slice32(1), write_vc->closed,
            write_vc->fragment);
-    if (!write_vc->alternate.valid())
+    if (!write_vc->alternate.valid()) {
       return openReadFromWriterFailure(CACHE_EVENT_OPEN_READ_FAILED, (Event *)-err);
+    }
     alternate.copy(&write_vc->alternate);
     vector.insert(&alternate);
     alternate.object_key_get(&key);
     write_vc->f.readers = 1;
     if (!(write_vc->f.update && write_vc->total_len == 0)) {
       key = write_vc->earliest_key;
-      if (!write_vc->closed)
+      if (!write_vc->closed) {
         alternate.object_size_set(write_vc->vio.nbytes);
-      else
+      } else {
         alternate.object_size_set(write_vc->total_len);
+      }
     } else {
       key = write_vc->update_key;
       ink_assert(write_vc->closed);
@@ -432,12 +445,9 @@ CacheVC::openReadFromWriter(int event, Event *e)
       return openReadStartEarliest(event, e);
     }
   } else {
-#endif // HTTP_CACHE
     DDebug("cache_read_agg", "%p: key: %X non-http passed stage 1", this, first_key.slice32(1));
     key = write_vc->earliest_key;
-#ifdef HTTP_CACHE
   }
-#endif
   if (write_vc->fragment) {
     doc_len        = write_vc->vio.nbytes;
     last_collision = nullptr;
@@ -476,8 +486,9 @@ CacheVC::openReadFromWriterMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNU
   }
   IOBufferBlock *b = nullptr;
   int64_t ntodo    = vio.ntodo();
-  if (ntodo <= 0)
+  if (ntodo <= 0) {
     return EVENT_CONT;
+  }
   if (length < ((int64_t)doc_len) - vio.ndone) {
     DDebug("cache_read_agg", "truncation %X", first_key.slice32(1));
     if (is_action_tag_set("cache")) {
@@ -493,8 +504,9 @@ CacheVC::openReadFromWriterMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNU
     iobufferblock_skip(writer_buf.get(), &writer_offset, &length, skip_bytes);
   }
   int64_t bytes = length;
-  if (bytes > vio.ntodo())
+  if (bytes > vio.ntodo()) {
     bytes = vio.ntodo();
+  }
   if (vio.ndone >= (int64_t)doc_len) {
     ink_assert(bytes <= 0);
     // reached the end of the document and the user still wants more
@@ -504,10 +516,11 @@ CacheVC::openReadFromWriterMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNU
   writer_buf = iobufferblock_skip(writer_buf.get(), &writer_offset, &length, bytes);
   vio.buffer.writer()->append_block(b);
   vio.ndone += bytes;
-  if (vio.ntodo() <= 0)
+  if (vio.ntodo() <= 0) {
     return calluser(VC_EVENT_READ_COMPLETE);
-  else
+  } else {
     return calluser(VC_EVENT_READ_READY);
+  }
 }
 
 int
@@ -515,17 +528,19 @@ CacheVC::openReadClose(int event, Event * /* e ATS_UNUSED */)
 {
   cancel_trigger();
   if (is_io_in_progress()) {
-    if (event != AIO_EVENT_DONE)
+    if (event != AIO_EVENT_DONE) {
       return EVENT_CONT;
+    }
     set_io_not_in_progress();
   }
   CACHE_TRY_LOCK(lock, vol->mutex, mutex->thread_holding);
-  if (!lock.is_locked())
+  if (!lock.is_locked()) {
     VC_SCHED_LOCK_RETRY();
+  }
   if (f.hit_evacuate && dir_valid(vol, &first_dir) && closed > 0) {
-    if (f.single_fragment)
+    if (f.single_fragment) {
       vol->force_evacuate_head(&first_dir, dir_pinned(&first_dir));
-    else if (dir_valid(vol, &earliest_dir)) {
+    } else if (dir_valid(vol, &earliest_dir)) {
       vol->force_evacuate_head(&first_dir, dir_pinned(&first_dir));
       vol->force_evacuate_head(&earliest_dir, dir_pinned(&earliest_dir));
     }
@@ -540,13 +555,15 @@ CacheVC::openReadReadDone(int event, Event *e)
   Doc *doc = nullptr;
 
   cancel_trigger();
-  if (event == EVENT_IMMEDIATE)
+  if (event == EVENT_IMMEDIATE) {
     return EVENT_CONT;
+  }
   set_io_not_in_progress();
   {
     CACHE_TRY_LOCK(lock, vol->mutex, mutex->thread_holding);
-    if (!lock.is_locked())
+    if (!lock.is_locked()) {
       VC_SCHED_LOCK_RETRY();
+    }
     if (event == AIO_EVENT_DONE && !io.ok()) {
       dir_delete(&earliest_key, vol, &earliest_dir);
       goto Lerror;
@@ -556,22 +573,26 @@ CacheVC::openReadReadDone(int event, Event *e)
     {
       doc = (Doc *)buf->data();
       if (doc->magic != DOC_MAGIC) {
-        char tmpstring[100];
-        if (doc->magic == DOC_CORRUPT)
+        char tmpstring[CRYPTO_HEX_SIZE];
+        if (doc->magic == DOC_CORRUPT) {
           Warning("Middle: Doc checksum does not match for %s", key.toHexStr(tmpstring));
-        else
+        } else {
           Warning("Middle: Doc magic does not match for %s", key.toHexStr(tmpstring));
+        }
         goto Lerror;
       }
-      if (doc->key == key)
+      if (doc->key == key) {
         goto LreadMain;
+      }
     }
-    if (last_collision && dir_offset(&dir) != dir_offset(last_collision))
+    if (last_collision && dir_offset(&dir) != dir_offset(last_collision)) {
       last_collision = nullptr; // object has been/is being overwritten
+    }
     if (dir_probe(&key, vol, &dir, &last_collision)) {
       int ret = do_read_call(&key);
-      if (ret == EVENT_RETURN)
+      if (ret == EVENT_RETURN) {
         goto Lcallreturn;
+      }
       return EVENT_CONT;
     } else if (write_vc) {
       if (writer_done()) {
@@ -597,10 +618,17 @@ CacheVC::openReadReadDone(int event, Event *e)
     }
     // fall through for truncated documents
   }
-Lerror:
-  char tmpstring[100];
-  Warning("Document %s truncated", earliest_key.toHexStr(tmpstring));
+Lerror : {
+  char tmpstring[CRYPTO_HEX_SIZE];
+  if (request.valid()) {
+    int url_length;
+    const char *url_text = request.url_get()->string_get_ref(&url_length);
+    Warning("Document %s truncated, url[%.*s]", earliest_key.toHexStr(tmpstring), url_length, url_text);
+  } else {
+    Warning("Document %s truncated", earliest_key.toHexStr(tmpstring));
+  }
   return calluser(VC_EVENT_ERROR);
+}
 Ldone:
   return calluser(VC_EVENT_EOS);
 Lcallreturn:
@@ -626,10 +654,9 @@ CacheVC::openReadMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
       vio.ndone = doc_len;
       return calluser(VC_EVENT_EOS);
     }
-#ifdef HTTP_CACHE
     HTTPInfo::FragOffset *frags = alternate.get_frag_table();
     if (is_debug_tag_set("cache_seek")) {
-      char b[33], c[33];
+      char b[CRYPTO_HEX_SIZE], c[CRYPTO_HEX_SIZE];
       Debug("cache_seek", "Seek @ %" PRId64 " in %s from #%d @ %" PRId64 "/%d:%s", seek_to, first_key.toHexStr(b), fragment,
             doc_pos, doc->len, doc->key.toHexStr(c));
     }
@@ -654,8 +681,9 @@ CacheVC::openReadMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
         while (seek_to >= next_off && target < lfi) {
           next_off = frags[++target];
         }
-        if (target == lfi && seek_to >= next_off)
+        if (target == lfi && seek_to >= next_off) {
           ++target;
+        }
       } else { // shortcut if we are in the fragment already
         target = fragment;
       }
@@ -674,7 +702,7 @@ CacheVC::openReadMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
         }
 
         if (is_debug_tag_set("cache_seek")) {
-          char target_key_str[33];
+          char target_key_str[CRYPTO_HEX_SIZE];
           key.toHexStr(target_key_str);
           Debug("cache_seek", "Seek #%d @ %" PRId64 " -> #%d @ %" PRId64 ":%s", cfi, doc_pos, target, seek_to, target_key_str);
         }
@@ -682,47 +710,92 @@ CacheVC::openReadMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
       }
     }
     doc_pos = doc->prefix_len() + seek_to;
-    if (fragment)
+    if (fragment && frags) {
       doc_pos -= static_cast<int64_t>(frags[fragment - 1]);
+    }
     vio.ndone = 0;
     seek_to   = 0;
     ntodo     = vio.ntodo();
     bytes     = doc->len - doc_pos;
     if (is_debug_tag_set("cache_seek")) {
-      char target_key_str[33];
-      key.toHexStr(target_key_str);
-      Debug("cache_seek", "Read # %d @ %" PRId64 "/%d for %" PRId64, fragment, doc_pos, doc->len, bytes);
+      char target_key_str[CRYPTO_HEX_SIZE];
+      Debug("cache_seek", "Read # %d @ %" PRId64 "/%d for %" PRId64 " %s", fragment, doc_pos, doc->len, bytes,
+            key.toHexStr(target_key_str));
+    }
+
+    // This shouldn't happen for HTTP assets but it does
+    // occasionally in production. This is a temporary fix
+    // to clean up broken objects until the root cause can
+    // be found. It must be the case that either the fragment
+    // offsets are incorrect or a fragment table isn't being
+    // created when it should be.
+    if (frag_type == CACHE_FRAG_TYPE_HTTP && bytes < 0) {
+      char xt[CRYPTO_HEX_SIZE];
+      char yt[CRYPTO_HEX_SIZE];
+
+      int url_length       = 0;
+      char const *url_text = nullptr;
+      if (request.valid()) {
+        url_text = request.url_get()->string_get_ref(&url_length);
+      }
+
+      int64_t prev_frag_size = 0;
+      if (fragment && frags) {
+        prev_frag_size = static_cast<int64_t>(frags[fragment - 1]);
+      }
+
+      Warning("cache_seek range request bug: read %s targ %s - %s frag # %d (prev_frag %" PRId64 ") @ %" PRId64 "/%d for %" PRId64
+              " tot %" PRId64 " url '%.*s'",
+              doc->key.toHexStr(xt), key.toHexStr(yt), f.single_fragment ? "single" : "multi", fragment, prev_frag_size, doc_pos,
+              doc->len, bytes, doc->total_len, url_length, url_text);
+
+      doc->magic = DOC_CORRUPT;
+
+      CACHE_TRY_LOCK(lock, vol->mutex, mutex->thread_holding);
+      if (!lock.is_locked()) {
+        SET_HANDLER(&CacheVC::openReadDirDelete);
+        VC_SCHED_LOCK_RETRY();
+      }
+
+      dir_delete(&earliest_key, vol, &earliest_dir);
+      goto Lerror;
     }
-#endif
   }
-  if (ntodo <= 0)
+  if (ntodo <= 0) {
     return EVENT_CONT;
-  if (vio.buffer.writer()->max_read_avail() > vio.buffer.writer()->water_mark && vio.ndone) // initiate read of first block
+  }
+  if (vio.buffer.writer()->max_read_avail() > vio.buffer.writer()->water_mark && vio.ndone) { // initiate read of first block
     return EVENT_CONT;
-  if ((bytes <= 0) && vio.ntodo() >= 0)
+  }
+  if ((bytes <= 0) && vio.ntodo() >= 0) {
     goto Lread;
-  if (bytes > vio.ntodo())
-    bytes     = vio.ntodo();
+  }
+  if (bytes > vio.ntodo()) {
+    bytes = vio.ntodo();
+  }
   b           = new_IOBufferBlock(buf, bytes, doc_pos);
   b->_buf_end = b->_end;
   vio.buffer.writer()->append_block(b);
   vio.ndone += bytes;
   doc_pos += bytes;
-  if (vio.ntodo() <= 0)
+  if (vio.ntodo() <= 0) {
     return calluser(VC_EVENT_READ_COMPLETE);
-  else {
-    if (calluser(VC_EVENT_READ_READY) == EVENT_DONE)
+  } else {
+    if (calluser(VC_EVENT_READ_READY) == EVENT_DONE) {
       return EVENT_DONE;
+    }
     // we have to keep reading until we give the user all the
     // bytes it wanted or we hit the watermark.
-    if (vio.ntodo() > 0 && !vio.buffer.writer()->high_water())
+    if (vio.ntodo() > 0 && !vio.buffer.writer()->high_water()) {
       goto Lread;
+    }
     return EVENT_CONT;
   }
 Lread : {
-  if (vio.ndone >= (int64_t)doc_len)
+  if (vio.ndone >= (int64_t)doc_len) {
     // reached the end of the document and the user still wants more
     return calluser(VC_EVENT_EOS);
+  }
   last_collision    = nullptr;
   writer_lock_retry = 0;
   // if the state machine calls reenable on the callback from the cache,
@@ -738,8 +811,9 @@ Lread : {
   if (dir_probe(&key, vol, &dir, &last_collision)) {
     SET_HANDLER(&CacheVC::openReadReadDone);
     int ret = do_read_call(&key);
-    if (ret == EVENT_RETURN)
+    if (ret == EVENT_RETURN) {
       goto Lcallreturn;
+    }
     return EVENT_CONT;
   } else if (write_vc) {
     if (writer_done()) {
@@ -758,8 +832,9 @@ Lread : {
     SET_HANDLER(&CacheVC::openReadMain);
     VC_SCHED_WRITER_RETRY();
   }
-  if (is_action_tag_set("cache"))
+  if (is_action_tag_set("cache")) {
     ink_release_assert(false);
+  }
   Warning("Document %X truncated at %d of %d, missing fragment %X", first_key.slice32(1), (int)vio.ndone, (int)doc_len,
           key.slice32(1));
   // remove the directory entry
@@ -784,34 +859,40 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
   Doc *doc = nullptr;
   cancel_trigger();
   set_io_not_in_progress();
-  if (_action.cancelled)
+  if (_action.cancelled) {
     return free_CacheVC(this);
+  }
   {
     CACHE_TRY_LOCK(lock, vol->mutex, mutex->thread_holding);
-    if (!lock.is_locked())
+    if (!lock.is_locked()) {
       VC_SCHED_LOCK_RETRY();
-    if (!buf)
+    }
+    if (!buf) {
       goto Lread;
-    if (!io.ok())
+    }
+    if (!io.ok()) {
       goto Ldone;
+    }
     // an object needs to be outside the aggregation window in order to be
     // be evacuated as it is read
     if (!dir_agg_valid(vol, &dir)) {
       // a directory entry which is nolonger valid may have been overwritten
-      if (!dir_valid(vol, &dir))
+      if (!dir_valid(vol, &dir)) {
         last_collision = nullptr;
+      }
       goto Lread;
     }
     doc = (Doc *)buf->data();
     if (doc->magic != DOC_MAGIC) {
-      char tmpstring[100];
+      char tmpstring[CRYPTO_HEX_SIZE];
       if (is_action_tag_set("cache")) {
         ink_release_assert(false);
       }
-      if (doc->magic == DOC_CORRUPT)
+      if (doc->magic == DOC_CORRUPT) {
         Warning("Earliest: Doc checksum does not match for %s", key.toHexStr(tmpstring));
-      else
+      } else {
         Warning("Earliest : Doc magic does not match for %s", key.toHexStr(tmpstring));
+      }
       // remove the dir entry
       dir_delete(&key, vol, &dir);
       // try going through the directory entries again
@@ -821,8 +902,9 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
       last_collision = nullptr;
       goto Lread;
     }
-    if (!(doc->key == key)) // collisiion
+    if (!(doc->key == key)) { // collisiion
       goto Lread;
+    }
     // success
     earliest_key = key;
     doc_pos      = doc->prefix_len();
@@ -831,20 +913,20 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
     if (vol->within_hit_evacuate_window(&earliest_dir) &&
         (!cache_config_hit_evacuate_size_limit || doc_len <= (uint64_t)cache_config_hit_evacuate_size_limit)) {
       DDebug("cache_hit_evac", "dir: %" PRId64 ", write: %" PRId64 ", phase: %d", dir_offset(&earliest_dir),
-             offset_to_vol_offset(vol, vol->header->write_pos), vol->header->phase);
+             vol->offset_to_vol_offset(vol->header->write_pos), vol->header->phase);
       f.hit_evacuate = 1;
     }
     goto Lsuccess;
   Lread:
     if (dir_probe(&key, vol, &earliest_dir, &last_collision) || dir_lookaside_probe(&key, vol, &earliest_dir, nullptr)) {
       dir = earliest_dir;
-      if ((ret = do_read_call(&key)) == EVENT_RETURN)
+      if ((ret = do_read_call(&key)) == EVENT_RETURN) {
         goto Lcallreturn;
+      }
       return ret;
     }
-// read has detected that alternate does not exist in the cache.
-// rewrite the vector.
-#ifdef HTTP_CACHE
+    // read has detected that alternate does not exist in the cache.
+    // rewrite the vector.
     if (!f.read_from_writer_called && frag_type == CACHE_FRAG_TYPE_HTTP) {
       // don't want any writers while we are evacuating the vector
       if (!vol->open_write(this, false, 1)) {
@@ -890,17 +972,18 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
             dir_set_tag(&od->single_doc_dir, od->single_doc_key.slice32(2));
           }
           SET_HANDLER(&CacheVC::openReadVecWrite);
-          if ((ret = do_write_call()) == EVENT_RETURN)
+          if ((ret = do_write_call()) == EVENT_RETURN) {
             goto Lcallreturn;
+          }
           return ret;
         }
       }
     }
-#endif
   // open write failure - another writer, so don't modify the vector
   Ldone:
-    if (od)
+    if (od) {
       vol->close_write(this);
+    }
   }
   CACHE_INCREMENT_DYN_STAT(cache_read_failure_stat);
   _action.continuation->handleEvent(CACHE_EVENT_OPEN_READ_FAILED, (void *)-ECACHE_NO_DOC);
@@ -908,15 +991,15 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
 Lcallreturn:
   return handleEvent(AIO_EVENT_DONE, nullptr); // hopefully a tail call
 Lsuccess:
-  if (write_vc)
+  if (write_vc) {
     CACHE_INCREMENT_DYN_STAT(cache_read_busy_success_stat);
+  }
   SET_HANDLER(&CacheVC::openReadMain);
   return callcont(CACHE_EVENT_OPEN_READ);
 }
 
 // create the directory entry after the vector has been evacuated
 // the volume lock has been taken when this function is called
-#ifdef HTTP_CACHE
 int
 CacheVC::openReadVecWrite(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
 {
@@ -924,12 +1007,14 @@ CacheVC::openReadVecWrite(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */
   set_io_not_in_progress();
   ink_assert(od);
   od->writing_vec = false;
-  if (_action.cancelled)
+  if (_action.cancelled) {
     return openWriteCloseDir(EVENT_IMMEDIATE, nullptr);
+  }
   {
     CACHE_TRY_LOCK(lock, vol->mutex, mutex->thread_holding);
-    if (!lock.is_locked())
+    if (!lock.is_locked()) {
       VC_SCHED_LOCK_RETRY();
+    }
     if (io.ok()) {
       ink_assert(f.evac_vector);
       ink_assert(frag_type == CACHE_FRAG_TYPE_HTTP);
@@ -941,8 +1026,9 @@ CacheVC::openReadVecWrite(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */
       f.use_first_key = 0;
       vio.op          = VIO::READ;
       dir_overwrite(&first_key, vol, &dir, &od->first_dir);
-      if (od->move_resident_alt)
+      if (od->move_resident_alt) {
         dir_insert(&od->single_doc_key, vol, &od->single_doc_dir);
+      }
       int alt_ndx = HttpTransactCache::SelectFromAlternates(write_vector, &request, params);
       vol->close_write(this);
       if (alt_ndx >= 0) {
@@ -964,7 +1050,6 @@ Lrestart:
   SET_HANDLER(&CacheVC::openReadStartHead);
   return openReadStartHead(EVENT_IMMEDIATE, nullptr);
 }
-#endif
 
 /*
   This code follows CacheVC::openReadStartEarliest closely,
@@ -977,34 +1062,40 @@ CacheVC::openReadStartHead(int event, Event *e)
   Doc *doc     = nullptr;
   cancel_trigger();
   set_io_not_in_progress();
-  if (_action.cancelled)
+  if (_action.cancelled) {
     return free_CacheVC(this);
+  }
   {
     CACHE_TRY_LOCK(lock, vol->mutex, mutex->thread_holding);
-    if (!lock.is_locked())
+    if (!lock.is_locked()) {
       VC_SCHED_LOCK_RETRY();
-    if (!buf)
+    }
+    if (!buf) {
       goto Lread;
-    if (!io.ok())
+    }
+    if (!io.ok()) {
       goto Ldone;
+    }
     // an object needs to be outside the aggregation window in order to be
     // be evacuated as it is read
     if (!dir_agg_valid(vol, &dir)) {
       // a directory entry which is nolonger valid may have been overwritten
-      if (!dir_valid(vol, &dir))
+      if (!dir_valid(vol, &dir)) {
         last_collision = nullptr;
+      }
       goto Lread;
     }
     doc = (Doc *)buf->data();
     if (doc->magic != DOC_MAGIC) {
-      char tmpstring[100];
+      char tmpstring[CRYPTO_HEX_SIZE];
       if (is_action_tag_set("cache")) {
         ink_release_assert(false);
       }
-      if (doc->magic == DOC_CORRUPT)
+      if (doc->magic == DOC_CORRUPT) {
         Warning("Head: Doc checksum does not match for %s", key.toHexStr(tmpstring));
-      else
+      } else {
         Warning("Head : Doc magic does not match for %s", key.toHexStr(tmpstring));
+      }
       // remove the dir entry
       dir_delete(&key, vol, &dir);
       // try going through the directory entries again
@@ -1014,18 +1105,20 @@ CacheVC::openReadStartHead(int event, Event *e)
       last_collision = nullptr;
       goto Lread;
     }
-    if (!(doc->first_key == key))
+    if (!(doc->first_key == key)) {
       goto Lread;
-    if (f.lookup)
+    }
+    if (f.lookup) {
       goto Lookup;
+    }
     earliest_dir = dir;
-#ifdef HTTP_CACHE
     CacheHTTPInfo *alternate_tmp;
     if (frag_type == CACHE_FRAG_TYPE_HTTP) {
       uint32_t uml;
       ink_assert(doc->hlen);
-      if (!doc->hlen)
+      if (!doc->hlen) {
         goto Ldone;
+      }
       if ((uml = this->load_http_info(&vector, doc)) != doc->hlen) {
         if (buf) {
           HTTPCacheAlt *alt  = reinterpret_cast<HTTPCacheAlt *>(doc->hdr());
@@ -1034,14 +1127,15 @@ CacheVC::openReadStartHead(int event, Event *e)
           // by bad disk data - count should be the number of successfully unmarshalled alts.
           for (int32_t i = 0; i < vector.count(); ++i) {
             CacheHTTPInfo *info = vector.get(i);
-            if (info && info->m_alt)
+            if (info && info->m_alt) {
               alt_length += info->m_alt->m_unmarshal_len;
+            }
           }
           Note("OpenReadHead failed for cachekey %X : vector inconsistency - "
-               "unmarshalled %d expecting %d in %d (base=%d, ver=%d:%d) "
+               "unmarshalled %d expecting %d in %d (base=%zu, ver=%d:%d) "
                "- vector n=%d size=%d"
                "first alt=%d[%s]",
-               key.slice32(0), uml, doc->hlen, doc->len, sizeofDoc, doc->v_major, doc->v_minor, vector.count(), alt_length,
+               key.slice32(0), uml, doc->hlen, doc->len, sizeof(Doc), doc->v_major, doc->v_minor, vector.count(), alt_length,
                alt->m_magic,
                (CACHE_ALT_MAGIC_ALIVE == alt->m_magic ?
                   "alive" :
@@ -1057,9 +1151,10 @@ CacheVC::openReadStartHead(int event, Event *e)
           err = ECACHE_ALT_MISS;
           goto Ldone;
         }
-      } else
+      } else {
         alternate_index = 0;
-      alternate_tmp     = vector.get(alternate_index);
+      }
+      alternate_tmp = vector.get(alternate_index);
       if (!alternate_tmp->valid()) {
         if (buf) {
           Note("OpenReadHead failed for cachekey %X : alternate inconsistency", key.slice32(0));
@@ -1080,9 +1175,7 @@ CacheVC::openReadStartHead(int event, Event *e)
       } else {
         f.single_fragment = false;
       }
-    } else
-#endif
-    {
+    } else {
       next_CacheKey(&key, &doc->key);
       f.single_fragment = doc->single_fragment();
       doc_pos           = doc->prefix_len();
@@ -1090,25 +1183,21 @@ CacheVC::openReadStartHead(int event, Event *e)
     }
 
     if (is_debug_tag_set("cache_read")) { // amc debug
-      char xt[33], yt[33];
+      char xt[CRYPTO_HEX_SIZE], yt[CRYPTO_HEX_SIZE];
       Debug("cache_read", "CacheReadStartHead - read %s target %s - %s %d of %" PRId64 " bytes, %d fragments",
             doc->key.toHexStr(xt), key.toHexStr(yt), f.single_fragment ? "single" : "multi", doc->len, doc->total_len,
-#ifdef HTTP_CACHE
-            alternate.get_frag_offset_count()
-#else
-            0
-#endif
-              );
+            alternate.get_frag_offset_count());
     }
     // the first fragment might have been gc'ed. Make sure the first
     // fragment is there before returning CACHE_EVENT_OPEN_READ
-    if (!f.single_fragment)
+    if (!f.single_fragment) {
       goto Learliest;
+    }
 
     if (vol->within_hit_evacuate_window(&dir) &&
         (!cache_config_hit_evacuate_size_limit || doc_len <= (uint64_t)cache_config_hit_evacuate_size_limit)) {
       DDebug("cache_hit_evac", "dir: %" PRId64 ", write: %" PRId64 ", phase: %d", dir_offset(&dir),
-             offset_to_vol_offset(vol, vol->header->write_pos), vol->header->phase);
+             vol->offset_to_vol_offset(vol->header->write_pos), vol->header->phase);
       f.hit_evacuate = 1;
     }
 
@@ -1137,8 +1226,9 @@ CacheVC::openReadStartHead(int event, Event *e)
     if (dir_probe(&key, vol, &dir, &last_collision)) {
       first_dir = dir;
       int ret   = do_read_call(&key);
-      if (ret == EVENT_RETURN)
+      if (ret == EVENT_RETURN) {
         goto Lcallreturn;
+      }
       return ret;
     }
   }
@@ -1168,3 +1258,18 @@ Learliest:
   SET_HANDLER(&CacheVC::openReadStartEarliest);
   return openReadStartEarliest(event, e);
 }
+
+/*
+   Handle a directory delete event in case of some detected corruption.
+*/
+int
+CacheVC::openReadDirDelete(int event, Event *e)
+{
+  MUTEX_TRY_LOCK(lock, vol->mutex, mutex->thread_holding);
+  if (!lock.is_locked()) {
+    VC_SCHED_LOCK_RETRY();
+  }
+
+  dir_delete(&earliest_key, vol, &earliest_dir);
+  return calluser(VC_EVENT_ERROR);
+}
