@@ -28,10 +28,12 @@
 #endif
 #define ROUNDUP(x, y) ((((x) + ((y)-1)) / (y)) * (y))
 
-using NetAcceptHandler      = int (NetAccept::*)(int, void *);
-volatile int dummy_volatile = 0;
-int accept_till_done        = 1;
+using NetAcceptHandler = int (NetAccept::*)(int, void *);
+int accept_till_done   = 1;
 
+// we need to protect naVec since it might be accessed
+// in different threads at the same time
+Ptr<ProxyMutex> naVecMutex;
 std::vector<NetAccept *> naVec;
 static void
 safe_delay(int msec)
@@ -62,12 +64,13 @@ net_accept(NetAccept *na, void *ep, bool blockable)
   // added by YTS Team, yamsat
   do {
     if ((res = na->server.accept(&con)) < 0) {
-      if (res == -EAGAIN || res == -ECONNABORTED || res == -EPIPE)
+      if (res == -EAGAIN || res == -ECONNABORTED || res == -EPIPE) {
         goto Ldone;
+      }
       if (na->server.fd != NO_FD && !na->action_->cancelled) {
-        if (!blockable)
+        if (!blockable) {
           na->action_->continuation->handleEvent(EVENT_ERROR, (void *)(intptr_t)res);
-        else {
+        } else {
           SCOPED_MUTEX_LOCK(lock, na->action_->mutex, e->ethread);
           na->action_->continuation->handleEvent(EVENT_ERROR, (void *)(intptr_t)res);
         }
@@ -75,10 +78,12 @@ net_accept(NetAccept *na, void *ep, bool blockable)
       count = res;
       goto Ldone;
     }
+    NET_SUM_GLOBAL_DYN_STAT(net_tcp_accept_stat, 1);
 
     vc = static_cast<UnixNetVConnection *>(na->getNetProcessor()->allocate_vc(e->ethread));
-    if (!vc)
+    if (!vc) {
       goto Ldone; // note: @a con will clean up the socket when it goes out of scope.
+    }
 
     ++count;
     NET_SUM_GLOBAL_DYN_STAT(net_connections_currently_open_stat, 1);
@@ -88,24 +93,37 @@ net_accept(NetAccept *na, void *ep, bool blockable)
     vc->mutex       = new_ProxyMutex();
     vc->action_     = *na->action_;
     vc->set_is_transparent(na->opt.f_inbound_transparent);
+    vc->set_is_proxy_protocol(na->opt.f_proxy_protocol);
     vc->set_context(NET_VCONNECTION_IN);
+    if (na->opt.f_mptcp) {
+      vc->set_mptcp_state(); // Try to get the MPTCP state, and update accordingly
+    }
+#ifdef USE_EDGE_TRIGGER
+    // Set the vc as triggered and place it in the read ready queue later in case there is already data on the socket.
+    if (na->server.http_accept_filter) {
+      vc->read.triggered = 1;
+    }
+#endif
     SET_CONTINUATION_HANDLER(vc, (NetVConnHandler)&UnixNetVConnection::acceptEvent);
 
-    if (e->ethread->is_event_type(na->opt.etype))
+    if (e->ethread->is_event_type(na->opt.etype)) {
       vc->handleEvent(EVENT_NONE, e);
-    else
+    } else {
       eventProcessor.schedule_imm(vc, na->opt.etype);
+    }
   } while (loop);
 
 Ldone:
-  if (!blockable)
+  if (!blockable) {
     MUTEX_UNTAKE_LOCK(na->action_->mutex.get(), e->ethread);
+  }
   return count;
 }
 
 NetAccept *
 getNetAccept(int ID)
 {
+  SCOPED_MUTEX_LOCK(lock, naVecMutex, this_ethread());
   return naVec.at(ID);
 }
 
@@ -133,8 +151,9 @@ NetAccept::init_accept_loop(const char *thr_name)
 void
 NetAccept::init_accept(EThread *t)
 {
-  if (!t)
+  if (!t) {
     t = eventProcessor.assign_thread(opt.etype);
+  }
 
   if (!action_->continuation->mutex) {
     action_->continuation->mutex = t->mutex;
@@ -161,13 +180,14 @@ NetAccept::init_accept_per_thread()
     return;
   }
 
-  if (accept_fn == net_accept)
+  if (accept_fn == net_accept) {
     SET_HANDLER((NetAcceptHandler)&NetAccept::acceptFastEvent);
-  else
+  } else {
     SET_HANDLER((NetAcceptHandler)&NetAccept::acceptEvent);
+  }
 
   period = -HRTIME_MSECONDS(net_accept_period);
-  n      = eventProcessor.n_threads_for_type[opt.etype];
+  n      = eventProcessor.thread_group[opt.etype]._count;
 
   for (i = 0; i < n; i++) {
     NetAccept *a;
@@ -178,17 +198,27 @@ NetAccept::init_accept_per_thread()
       a = this;
     }
 
-    EThread *t         = eventProcessor.eventthread[opt.etype][i];
+    EThread *t         = eventProcessor.thread_group[opt.etype]._thread[i];
     PollDescriptor *pd = get_PollDescriptor(t);
 
-    if (a->ep.start(pd, a, EVENTIO_READ) < 0)
+    if (a->ep.start(pd, a, EVENTIO_READ) < 0) {
       Warning("[NetAccept::init_accept_per_thread]:error starting EventIO");
+    }
 
     a->mutex = get_NetHandler(t)->mutex;
     t->schedule_every(a, period, opt.etype);
   }
 }
 
+void
+NetAccept::stop_accept()
+{
+  if (!action_->cancelled) {
+    action_->cancel();
+  }
+  server.close();
+}
+
 int
 NetAccept::do_listen(bool non_blocking)
 {
@@ -207,10 +237,11 @@ NetAccept::do_listen(bool non_blocking)
   }
 
   if (opt.f_callback_on_open && !action_->cancelled) {
-    if (res)
+    if (res) {
       action_->continuation->handleEvent(NET_EVENT_ACCEPT_FAILED, this);
-    else
+    } else {
       action_->continuation->handleEvent(NET_EVENT_ACCEPT_SUCCEED, this);
+    }
     mutex = nullptr;
   }
 
@@ -228,12 +259,12 @@ NetAccept::do_blocking_accept(EThread *t)
   // do-while for accepting all the connections
   // added by YTS Team, yamsat
   do {
-    ink_hrtime now = Thread::get_hrtime();
     if ((res = server.accept(&con)) < 0) {
       int seriousness = accept_error_seriousness(res);
       if (seriousness >= 0) { // not so bad
-        if (!seriousness)     // bad enough to warn about
+        if (!seriousness) {   // bad enough to warn about
           check_transient_accept_error(res);
+        }
         safe_delay(net_throttle_delay);
         return 0;
       }
@@ -253,25 +284,40 @@ NetAccept::do_blocking_accept(EThread *t)
       continue;
     }
 
+    if (shutdown_event_system == true) {
+      return -1;
+    }
+
     // Use 'nullptr' to Bypass thread allocator
     vc = (UnixNetVConnection *)this->getNetProcessor()->allocate_vc(nullptr);
-    if (unlikely(!vc || shutdown_event_system == true)) {
+    if (unlikely(!vc)) {
       return -1;
     }
 
     NET_SUM_GLOBAL_DYN_STAT(net_connections_currently_open_stat, 1);
+    NET_SUM_GLOBAL_DYN_STAT(net_tcp_accept_stat, 1);
     vc->id = net_next_connection_number();
     vc->con.move(con);
-    vc->submit_time = now;
+    vc->submit_time = Thread::get_hrtime();
     vc->mutex       = new_ProxyMutex();
     vc->action_     = *action_;
     vc->set_is_transparent(opt.f_inbound_transparent);
+    vc->set_is_proxy_protocol(opt.f_proxy_protocol);
     vc->options.packet_mark = opt.packet_mark;
     vc->options.packet_tos  = opt.packet_tos;
     vc->options.ip_family   = opt.ip_family;
     vc->apply_options();
     vc->set_context(NET_VCONNECTION_IN);
+    if (opt.f_mptcp) {
+      vc->set_mptcp_state(); // Try to get the MPTCP state, and update accordingly
+    }
     vc->accept_object = this;
+#ifdef USE_EDGE_TRIGGER
+    // Set the vc as triggered and place it in the read ready queue later in case there is already data on the socket.
+    if (server.http_accept_filter) {
+      vc->read.triggered = 1;
+    }
+#endif
     SET_CONTINUATION_HANDLER(vc, (NetVConnHandler)&UnixNetVConnection::acceptEvent);
     // eventProcessor.schedule_imm(vc, getEtype());
     eventProcessor.schedule_imm_signal(vc, opt.etype);
@@ -332,7 +378,6 @@ NetAccept::acceptFastEvent(int event, void *ep)
   int bufsz, res = 0;
   Connection con;
 
-  PollDescriptor *pd     = get_PollDescriptor(e->ethread);
   UnixNetVConnection *vc = nullptr;
   int loop               = accept_till_done;
 
@@ -348,12 +393,14 @@ NetAccept::acceptFastEvent(int event, void *ep)
 
     if (likely(fd >= 0)) {
       Debug("iocore_net", "accepted a new socket: %d", fd);
+      NET_SUM_GLOBAL_DYN_STAT(net_tcp_accept_stat, 1);
       if (opt.send_bufsize > 0) {
         if (unlikely(socketManager.set_sndbuf_size(fd, opt.send_bufsize))) {
           bufsz = ROUNDUP(opt.send_bufsize, 1024);
           while (bufsz > 0) {
-            if (!socketManager.set_sndbuf_size(fd, bufsz))
+            if (!socketManager.set_sndbuf_size(fd, bufsz)) {
               break;
+            }
             bufsz -= 1024;
           }
         }
@@ -362,8 +409,9 @@ NetAccept::acceptFastEvent(int event, void *ep)
         if (unlikely(socketManager.set_rcvbuf_size(fd, opt.recv_bufsize))) {
           bufsz = ROUNDUP(opt.recv_bufsize, 1024);
           while (bufsz > 0) {
-            if (!socketManager.set_rcvbuf_size(fd, bufsz))
+            if (!socketManager.set_rcvbuf_size(fd, bufsz)) {
               break;
+            }
             bufsz -= 1024;
           }
         }
@@ -388,14 +436,15 @@ NetAccept::acceptFastEvent(int event, void *ep)
 #if defined(linux)
           || res == -EPIPE
 #endif
-          ) {
+      ) {
         goto Ldone;
       } else if (accept_error_seriousness(res) >= 0) {
         check_transient_accept_error(res);
         goto Ldone;
       }
-      if (!action_->cancelled)
+      if (!action_->cancelled) {
         action_->continuation->handleEvent(EVENT_ERROR, (void *)(intptr_t)res);
+      }
       goto Lerror;
     }
 
@@ -411,41 +460,28 @@ NetAccept::acceptFastEvent(int event, void *ep)
     vc->mutex       = new_ProxyMutex();
     // no need to set vc->action_
     vc->set_is_transparent(opt.f_inbound_transparent);
+    vc->set_is_proxy_protocol(opt.f_proxy_protocol);
     vc->options.packet_mark = opt.packet_mark;
     vc->options.packet_tos  = opt.packet_tos;
     vc->options.ip_family   = opt.ip_family;
     vc->apply_options();
     vc->set_context(NET_VCONNECTION_IN);
-    SET_CONTINUATION_HANDLER(vc, (NetVConnHandler)&UnixNetVConnection::mainEvent);
-
-    // set thread and nh as acceptEvent does
-    vc->thread = e->ethread;
-    vc->nh     = get_NetHandler(e->ethread);
-
-    if (vc->ep.start(pd, vc, EVENTIO_READ | EVENTIO_WRITE) < 0) {
-      Warning("[NetAccept::acceptFastEvent]: Error in inserting fd[%d] in kevent\n", vc->con.fd);
-      close_UnixNetVConnection(vc, e->ethread);
-      return EVENT_DONE;
+    vc->action_ = *action_;
+    if (opt.f_mptcp) {
+      vc->set_mptcp_state(); // Try to get the MPTCP state, and update accordingly
     }
 
-    ink_assert(vc->nh->mutex->thread_holding == this_ethread());
-    vc->set_inactivity_timeout(0);
-    vc->nh->open_list.enqueue(vc);
-
 #ifdef USE_EDGE_TRIGGER
-    // Set the vc as triggered and place it in the read ready queue in case there is already data on the socket.
-    Debug("iocore_net", "acceptEvent : Setting triggered and adding to the read ready queue");
-    vc->read.triggered = 1;
-    vc->nh->read_ready_list.enqueue(vc);
-#endif
-
-    if (!action_->cancelled) {
-      // We must be holding the lock already to do later do_io_read's
-      SCOPED_MUTEX_LOCK(lock, vc->mutex, e->ethread);
-      action_->continuation->handleEvent(NET_EVENT_ACCEPT, vc);
-    } else {
-      close_UnixNetVConnection(vc, e->ethread);
+    // Set the vc as triggered and place it in the read ready queue later in case there is already data on the socket.
+    if (server.http_accept_filter) {
+      vc->read.triggered = 1;
     }
+#endif
+    SET_CONTINUATION_HANDLER(vc, (NetVConnHandler)&UnixNetVConnection::acceptEvent);
+    // We must be holding the lock already to do later do_io_read's
+    SCOPED_MUTEX_LOCK(lock, vc->mutex, e->ethread);
+    vc->handleEvent(EVENT_NONE, nullptr);
+    vc = nullptr;
   } while (loop);
 
 Ldone:
@@ -454,8 +490,6 @@ Ldone:
 Lerror:
   server.close();
   e->cancel();
-  if (vc)
-    vc->free(e->ethread);
   NET_DECREMENT_DYN_STAT(net_accepts_currently_open_stat);
   delete this;
   return EVENT_DONE;
@@ -468,8 +502,9 @@ NetAccept::acceptLoopEvent(int event, Event *e)
   (void)e;
   EThread *t = this_ethread();
 
-  while (do_blocking_accept(t) >= 0)
+  while (do_blocking_accept(t) >= 0) {
     ;
+  }
 
   // Don't think this ever happens ...
   NET_DECREMENT_DYN_STAT(net_accepts_currently_open_stat);
@@ -482,9 +517,7 @@ NetAccept::acceptLoopEvent(int event, Event *e)
 //
 //
 
-NetAccept::NetAccept(const NetProcessor::AcceptOptions &_opt) : Continuation(nullptr), opt(_opt)
-{
-}
+NetAccept::NetAccept(const NetProcessor::AcceptOptions &_opt) : Continuation(nullptr), opt(_opt) {}
 
 //
 // Stop listening.  When the next poll takes place, an error will result.
